@@ -1,33 +1,27 @@
-#################################################################################
+## Copyright (C) 2012 Marius Hofert, Ivan Kojadinovic, Martin Maechler, and Jun Yan
 ##
-##   R package Copula by Jun Yan and Ivan Kojadinovic Copyright (C) 2009
+## This program is free software; you can redistribute it and/or modify it under
+## the terms of the GNU General Public License as published by the Free Software
+## Foundation; either version 3 of the License, or (at your option) any later
+## version.
 ##
-##   This file is part of the R package copula.
+## This program is distributed in the hope that it will be useful, but WITHOUT
+## ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+## FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
+## details.
 ##
-##   The R package copula is free software: you can redistribute it and/or modify
-##   it under the terms of the GNU General Public License as published by
-##   the Free Software Foundation, either version 3 of the License, or
-##   (at your option) any later version.
-##
-##   The R package copula is distributed in the hope that it will be useful,
-##   but WITHOUT ANY WARRANTY; without even the implied warranty of
-##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-##   GNU General Public License for more details.
-##
-##   You should have received a copy of the GNU General Public License
-##   along with the R package copula. If not, see <http://www.gnu.org/licenses/>.
-##
-#################################################################################
+## You should have received a copy of the GNU General Public License along with
+## this program; if not, see <http://www.gnu.org/licenses/>.
 
 
-indepCopula <- function(dim = 2) {
+indepCopula <- function(dim = 2L) {
     ## get expressions of cdf and pdf
   cdfExpr <- function(n) {
     uis <- paste("u", 1:n, sep="")
     expr <- paste(uis, collapse="*")
     parse(text = expr)
   }
-  
+
   pdfExpr <- function(cdf, n) {
     val <- cdf
     for (i in 1:n) {
@@ -35,10 +29,10 @@ indepCopula <- function(dim = 2) {
     }
     val
   }
-  cdf <- cdfExpr(dim)
+  cdf <- cdfExpr((dim <- as.integer(dim)))
   pdf <- pdfExpr(cdf, dim)
-  
-  val <- new("indepCopula",
+
+  new("indepCopula",
              dimension = dim,
              exprdist = c(cdf=cdf, pdf=pdf),
              parameters = double(0),
@@ -46,10 +40,8 @@ indepCopula <- function(dim = 2) {
              param.lowbnd = double(0),
              param.upbnd = double(0),
              message = "Independence copula")
-  val
 }
 
-
 AfunIndep <- function(copula, w) {
   rep(1, length(w))
 }
@@ -59,16 +51,16 @@ rindepCopula <- function(copula, n) {
   matrix(runif(n * dim), nrow = n)
 }
 
-pindepCopula <- function(copula, u) {
-  if (is.vector(u)) u <- matrix(u, nrow = 1)
+pindepCopula <- function(copula, u, log.p=FALSE) {
+  if(!is.matrix(u)) u <- rbind(u, deparse.level = 0L)
   stopifnot (ncol(u) == copula@dimension)
-  apply(u, 1, prod)
+  if(log.p) rowSums(log(u)) else apply(u, 1, prod)
 }
 
-dindepCopula <- function(copula, u) {
-  if (is.vector(u)) u <- matrix(u, nrow = 1)
+dindepCopula <- function(copula, u, log=FALSE, ...) {
+  if(!is.matrix(u)) u <- rbind(u, deparse.level = 0L)
   stopifnot (ncol(u) == copula@dimension)
-  rep(1, nrow(u))
+  rep.int(if(log) 0 else 1, nrow(u))
 }
 
 ## kendallsTauIndepCopula <- function(copula) {
